#include "Laser.h"
#include "../Karte/Karte.h"
#include "../Spieler/Spieler.h"
#include "../Define.h"

// Inhalt der Laser Klasse aus Laser.h
// Konstruktor
Laser::Laser( int id, Vec2< double > pos, Vec2< double > speed, int sNum, double intensitt, int tf )
    : Object2D()
{
    this->id = id;
    setPosition( pos );
    setSpeed( speed );
    this->sNum = sNum;
    this->intensitt = intensitt;
    this->startIntensitt = intensitt;
    this->tf = tf;
}

// privat
char Laser::getOutCode( Punkt& p, Bild &zrObj ) const
{
    char ret = 0;
    if( p.x < 0 )
        ret |= 1;
    else if( p.x >= zrObj.getBreite() )
        ret |= 2;
    if( p.y < 0 )
        ret |= 4;
    else if( p.y >= zrObj.getHeight() )
        ret |= 8;
    return ret;
}

// nicht constant
Rect2< float > Laser::getBoundingBox() const
{
    return Rect2< float >();
}

bool Laser::tick( const WeltInfo &info, double tv )
{
    __super::tick( info, tv );
    intensitt -= tv * 2;
    return 1;
}

void Laser::render( Mat3< float > &kamMat, Bild &zRObj )
{
    int fa = (int)( ( intensitt / startIntensitt ) * 255 );
    int f = ( ( fa << 24 ) & 0xFF000000 ) | ( tf & 0xFFFFFF );
    Punkt a = (Punkt)( kamMat * getPosition() );
    Punkt b = (Punkt)( kamMat * ( getPosition() - ( getSpeed() / getSpeed().getLength() * 10 ) ) );
    a += zRObj.getDrawOff();
    b += zRObj.getDrawOff();
    char outCode1 = getOutCode( a, zRObj );
    char outCode2 = getOutCode( b, zRObj );
    bool ok = 0;
    while( 1 )
    {
        int xMax = zRObj.getDrawGr().x - 1;
        int yMax = zRObj.getDrawGr().y - 1;
        if( !( outCode1 | outCode2 ) )
        {
            ok = 1;
            break;
        }
        else if( outCode1 & outCode2 )
            break;
        else
        {
            int x = 0, y = 0;
            char outCodeOut = outCode1 ? outCode1 : outCode2;
            if( outCodeOut & 8 )
            {
                x = (int)( a.x + ( b.x - a.x ) * ( yMax - a.y ) / ( b.y - a.y ) + 0.5 );
                y = yMax;
            }
            else if( outCodeOut & 4 )
            {
                x = (int)( a.x + ( b.x - a.x ) * ( zRObj.getDrawPos().y - a.y ) / ( b.y - a.y ) + 0.5 );
                y = zRObj.getDrawPos().y;
            }
            else if( outCodeOut & 2 )
            {
                y = (int)( a.y + ( b.y - a.y ) * ( xMax - a.x ) / ( b.x - a.x ) + 0.5 );
                x = xMax;
            }
            else if( outCodeOut & 1 )
            {
                y = (int)( a.y + ( b.y - a.y ) * ( zRObj.getDrawPos().x - a.x ) / ( b.x - a.x ) + 0.5 );
                x = zRObj.getDrawPos().x;
            }
            if( outCodeOut == outCode1 )
            {
                a.x = x;
                a.y = y;
                outCode1 = getOutCode( a, zRObj );
            }
            else
            {
                b.x = x;
                b.y = y;
                outCode2 = getOutCode( b, zRObj );
            }
        }
    }
    if( ok )
    {
        int xln = b.x - a.x, axln = abs( xln );
        int yln = b.y - a.y, ayln = abs( yln );
        double xf = (double)xln / ( ayln ? ayln : 1 );
        double yf = (double)yln / ( axln ? axln : 1 );
        if( axln > ayln )
            xf = xf < 0 ? -1 : 1;
        else
            yf = yf < 0 ? -1 : 1;
        double x = (double)a.x, y = (double)a.y;
        int maxP = (int)( sqrt( xln * xln + yln * yln ) + 0.5 );
        int count = 0;
        int *buffer = zRObj.getBuffer();
        int maxPixel = zRObj.getBreite() * zRObj.getHeight();
        while( !( (int)( x + 0.5 ) == b.x && (int)( y + 0.5 ) == b.y ) && count < maxP )
        {
            ++count;
            if( (int)( (int)( x + 0.5 ) + (int)( y + 0.5 ) * zRObj.getBreite() ) < maxPixel )
                buffer[ (int)( (int)( x + 0.5 ) + (int)( y + 0.5 ) * zRObj.getBreite() ) ] = 0xFFFFFFFF;
            if( (int)( (int)( x - 0.5 ) + (int)( y + 0.5 ) * zRObj.getBreite() ) < maxPixel &&
                buffer[ (int)( (int)( x - 0.5 ) + (int)( y + 0.5 ) * zRObj.getBreite() ) ] != 0xFFFFFFFF )
                zRObj.alphaPixel( (int)( x - 0.5 ), (int)( y + 0.5 ), f );
            if( (int)( (int)( x + 1.5 ) + (int)( y + 0.5 ) * zRObj.getBreite() ) < maxPixel &&
                buffer[ (int)( (int)( x + 1.5 ) + (int)( y + 0.5 ) * zRObj.getBreite() ) ] != 0xFFFFFFFF )
                zRObj.alphaPixel( (int)( x + 1.5 ), (int)( y + 0.5 ), f );
            if( (int)( (int)( x + 0.5 ) + (int)( y - 0.5 ) * zRObj.getBreite() ) < maxPixel &&
                buffer[ (int)( (int)( x + 0.5 ) + (int)( y - 0.5 ) * zRObj.getBreite() ) ] != 0xFFFFFFFF )
                zRObj.alphaPixel( (int)( x + 0.5 ), (int)( y - 0.5 ), f );
            if( (int)( (int)( x + 0.5 ) + (int)( y + 1.5 ) * zRObj.getBreite() ) < maxPixel &&
                buffer[ (int)( (int)( x + 0.5 ) + (int)( y + 1.5 ) * zRObj.getBreite() ) ] != 0xFFFFFFFF )
                zRObj.alphaPixel( (int)( x + 0.5 ), (int)( y + 1.5 ), f );
            x += xf, y += yf;
        }
    }
}

// constant
int Laser::getId() const
{
    return id;
}

int Laser::getSpieler() const
{
    return sNum;
}

double Laser::getIntensitt( Vec2< double > targetSpeed ) const
{
    return intensitt * ( ( speed - targetSpeed ).getLength() / 200 );
}

// Reference Counting
Laser *Laser::getThis()
{
    ref++;
    return this;
}

Laser *Laser::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}